* RATS code to calculate orthogonalized IRF
* written by James Hamilton March 2016
cal 1954 3 4
      * identifies as quarterly observations beginning 1954:Q3
compute capt = 2015:4
      * a variable capt is introduced to facilitate updating with later data
open data var3_data.csv
data(org=obs) 1954:3 capt datex gdplev pce_def fedfunds
* col 1:  date
* col 2:  level of real GDP (2009 dollars)
* col 3:  level of implicit price deflator on personal consumption expenditures (2009 = 100)
* col 4:  average value of effective fed funds rate over the quarter

* convert GDP and price level to annual growth rates
set gdpch = 400*log(gdplev(t)/gdplev(t-1))
set inflation = 400*log(pce_def(t)/pce_def(t-1))

smpl 1954:3 2007:4

system(model=var4)
variables gdpch inflation fedfunds
lags 1 to 4
det constant
end(system)
estimate(sigma)

* ============================================================
* calculate A matrix

compute P = %decomp(%sigma)
compute X1 = %xdiag(P)
compute X2 = %diag(X1)
compute A = P*inv(X2)
display A

* =============================================================
* calculate orthogonal IRF using columns of A as shocks

compute neqn = 3
compute nsteps = 20

declare rect[series] impblk(neqn,neqn)
declare rect[series] imptemp(neqn,1)
declare vect[labels] implabel(neqn)
compute implabel=|| 'GDP', 'inflation', 'fed funds'||
declare vect[string] alllabel(neqn)
compute alllabel=|| 'GDP', 'inflation', 'fed funds'||
smpl 1 nsteps
do j=1,neqn
impulse(model=var4, shock=%xcol(A,j), result=imptemp) neqn nsteps
set impblk(1,j) = imptemp(1,1)
set impblk(2,j) = imptemp(2,1)
set impblk(3,j) = imptemp(3,1)
spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Response of ' +implabel(i) +' to ' +implabel(j)
graph(header=graphlabel,nodates,number=0) 1
# impblk(i,1)
end do i
spgraph(done)
end do j

* =================================================================
* calculate orthogonal IRF using local projection
declare rect[series] impjorda(neqn,neqn)
do k=1,nsteps
smpl 1954:3 2007:4
linreg gdpch
# gdpch{k-1}  constant gdpch{k to k+3} inflation{k to k+3} fedfunds{k to k+3}
compute a1 = %beta(1)
linreg inflation
# gdpch{k-1}  constant gdpch{k to k+3} inflation{k to k+3} fedfunds{k to k+3}
compute a2 = %beta(1)
linreg fedfunds
# gdpch{k-1}  constant gdpch{k to k+3} inflation{k to k+3} fedfunds{k to k+3}
compute a3 = %beta(1)
smpl 1 nsteps
set impjorda(1,1) k k = a1
set impjorda(2,1) k k = a2
set impjorda(3,1) k k = a3
end do
spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Response of ' +implabel(i) +' to GDP alone (local projection)'
graph(header=graphlabel,nodates)
# impjorda(i,1)
end do
spgraph(done)

spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Response of ' +implabel(i) +' to GDP alone (VAR)'
graph(header=graphlabel,nodates,number=0)
# impblk(i,1)
end do
spgraph(done)

spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Response of ' +implabel(i) +' to inflation given GDP (VAR)'
graph(header=graphlabel,nodates,number=0)
# impblk(i,2)
end do
spgraph(done)

print(nodates) 1 nsteps impjorda(1,1) impjorda(2,1) impjorda(3,1)

* ===============================================================================
* calculate variance decomposition
smpl 1954:3 2007:4

system(model=var4)
variables gdpch inflation fedfunds
lags 1 to 4
det constant
end(system)
estimate(sigma,noprint)

smpl 1 nsteps
errors(model=var4,result=impblk) neqn nsteps * %sigma

* =================================================================================
* calculate historical decomposition

smpl 1954:3 2007:4

system(model=var4)
variables gdpch inflation fedfunds
lags 1 to 4
det constant
end(system)
estimate(sigma,noprint)


compute hstart=1970:1
compute hend  =2007:4

history(model=var4,add,results=history,from=hstart,to=hend)
   spgraph(vfields=neqn, header="Historical Decomposition of GDP")
   do i=1,neqn
      * This graphs the actual data (including four pre-sample values),
      * the base forecast, and the effects of shock i on variable 1.
      *
      graph(header="Effect of "+implabel(i)) 3
      # gdpch hstart-4 hend
      # history(1,1)
      # history(1+i,1)
   end do i
   spgraph(done)

* =================================================================================
* calculate confidence intervals for VAR

smpl 1954:3 2007:4

system(model=var4)
variables gdpch inflation fedfunds
lags 1 to 4
det constant
end(system)
estimate(sigma,noprint)
source montevar95.src
@MonteVar95(model=var4, steps=nsteps,draws=10000,shocklabels=alllabel,varlabels=alllabel)



